/* $Id: PoolingConnection.java,v 3.1 2001/11/06 04:00:15 886014 Rel $
 * Copyright (c) 2001 TSUTSUMI Kikuo. ALL RIGHTS RESERVED.
 */
package connectionpool;

import java.sql.*;
import java.util.Timer;
import java.util.TimerTask;
import java.util.Map;

/**
 * <P>̃NX͈JDBCڑ̏ԂǗ܂Bgp񐔁Aڑ^C
 * AEgԂ̊Ǘs܂BConnection IuWFNg̃VEbp[
 * NXłB</P>
 *
 * <P>̃NX̓pbP[WXR[vł邽߁Aۂɂ API p҂
 * ڎgp邱Ƃ͂܂B</P>
 *
 * @author TSUTSUMI Kikuo
 * @version $Revision: 3.1 $
 **/
class PoolingConnection implements Connection
{
  /** ̐ڑIuWFNgێڑv[IuWFNg */
  private ConnectionPool owner;
  /** JDBCڑIuWFNg */
  private Connection conn = null;
  /** gpꂽ */
  private int useCount = 0;
  /** őgp */
  private final int maxUse;
  /** ڑ^CAEg(ms) */
  private final long timeout;
  /** ڑ^CAEgĎ^C}[ */
  private Timer timeoutTimer = null;
  /**
   * <P>JDBCڑIuWFNgi[A^CAEgԁAőgp񐔂
   * ݒ肵܂B</P>
   *
   * @param owner ̐ڑIuWFNgێڑv[IuWFNgB
   * @param conn JDBCڑIuWFNgB
   *
   * @throws SQLException ڑmłȂꍇB
   **/
  PoolingConnection (ConnectionPool owner, Connection conn)
    throws SQLException
  {
    this.owner = owner;
    this.conn = conn;
    useCount = 0;
    this.timeout = ConnectionPoolDriver.timeout;
    this.maxUse = ConnectionPoolDriver.maxUse;
    timeoutTimer = null;
    return;
  }
  /**
   * <P>ڑgp̏Ԃɂ܂B</P>
   *
   * @return gւ̎QƁB
   **/
  PoolingConnection activate ()
  {
    useCount ++;
    cancelTimer ();  // gp̓^CAEgĎȂ
    return this;
  }
  /**
   * <P>ڑ𖢎gp̏ԂɂAg𖢎gpڑv[ɓo^܂B
   * </P>
   *
   * <P>̎Agp񐔂őgp񐔂ɒBĂJDBCڑؒfA
   * ڑv[玩g폜܂B</P>
   *
   * @throws SQLException JDBCڑ̐ؒfɎsꍇB
   **/
  public void close() throws SQLException
  {
    if (maxUse >= 0 && useCount >= maxUse)	// őgp񐔂ɒB
      {
	removeConnection ();
      }
    else
      {
	setTimer ();				// ^CAEgĎJn
	owner.release (this);			// gpԂɂ
      }
  }
  /**
   * <P>JDBCڑؒf܂B̃\bhsƁÃv[O
   * ڑIuWFNg̖ƂȂ̂ŁAڑv[菜GC
   * j悤ɂȂ΂Ȃ܂B</P>
   *
   * @throws SQLException JDBCڑ̐ؒfɎsꍇB
   **/
  void closeConnection () throws SQLException
  {
    cancelTimer ();
    if (conn == null)
      return;
    conn.close ();
    conn = null;
    owner = null;
  }
  /**
   * <P>ڑv[炱̃IuWFNgg폜AJDBCڑؒf
   * B</P>
   *
   * @throws SQLException JDBCڑ̐ؒfɎsꍇB
   **/
  private void removeConnection () throws SQLException
  {
    // Ƀv[폜ĂȂƁAcloseConnection ŗO
    // ƂɃIuWFNg[NĂ܂B
    owner.remove (this);
    closeConnection ();
  }
  /**
   * <P>ڑ^CAEgĎ^C}[ݒ肵܂Bw肳ꂽ^CAE
   * gԂo߂ڑ폜܂B^CAEgԂ0ȉ̏ꍇ
   * ̓^C}[ݒ肵܂B</P>
   **/
  private void setTimer ()
  {
    if (timeoutTimer != null)
      timeoutTimer.cancel ();
    if (timeout < 1)
      {
	timeoutTimer = null;
	return;
      }
    timeoutTimer = new Timer ();
    timeoutTimer.schedule (new TimerTask () {
	public void run ()
	{
	  try
	    {
	      removeConnection ();
	    }
	  catch (SQLException e)
	    {
	      e.printStackTrace ();
	    }
	}
      }, timeout);
  }
  /**
   * <P>^C}[܂B</P>
   **/
  private void cancelTimer ()
  {
    if (timeoutTimer == null)
      return;
    timeoutTimer.cancel ();
    timeoutTimer = null;
  }
  /* fobOp */
  public String toString ()
  {
    StringBuffer str = new StringBuffer (conn.toString ());
    str.append ("; use: ");
    str.append (useCount);
    str.append ("; timer: ");
    str.append (timeoutTimer);
    return (str.toString ());
  }
  // implements methods
  public void clearWarnings() throws SQLException { conn.clearWarnings (); }
  public void commit() throws SQLException { conn.commit(); }
  public Statement createStatement() throws SQLException
  { return conn.createStatement(); }
  public Statement createStatement(int resultSetType,
				   int resultSetConcurrency)
    throws SQLException
  { return conn.createStatement(resultSetType, resultSetConcurrency); }
  public boolean getAutoCommit() throws SQLException
  { return conn.getAutoCommit (); }
  public String getCatalog() throws SQLException { return conn.getCatalog(); }
  public DatabaseMetaData getMetaData() throws SQLException
  { return conn.getMetaData (); }
  public int getTransactionIsolation() throws SQLException
  { return conn.getTransactionIsolation (); }
  public Map getTypeMap() throws SQLException { return conn.getTypeMap (); }
  public SQLWarning getWarnings() throws SQLException
  { return conn.getWarnings(); }
  public boolean isClosed() throws SQLException
  { return conn == null || conn.isClosed(); }
  public boolean isReadOnly() throws SQLException { return conn.isReadOnly(); }
  public String nativeSQL(String sql) throws SQLException
  { return conn.nativeSQL(sql); }
  public  CallableStatement prepareCall(String sql) throws SQLException
  { return conn.prepareCall(sql); }
  public CallableStatement prepareCall(String sql,
				       int resultSetType,
				       int resultSetConcurrency)
    throws SQLException
  { return conn.prepareCall(sql, resultSetType, resultSetConcurrency); }
  public PreparedStatement prepareStatement(String sql) throws SQLException
  { return conn.prepareStatement(sql); }
  public PreparedStatement prepareStatement(String sql,
					    int resultSetType,
					    int resultSetConcurrency)
    throws SQLException
  { return conn.prepareStatement(sql, resultSetType, resultSetConcurrency); }
  public void rollback() throws SQLException { conn.rollback(); }
  public  void setAutoCommit(boolean autoCommit) throws SQLException
  { conn.setAutoCommit(autoCommit); }
  public void setCatalog(String catalog) throws SQLException
  { conn.setCatalog(catalog); }
  public void setReadOnly(boolean readOnly) throws SQLException
  { conn.setReadOnly(readOnly); }
  public void setTransactionIsolation(int level) throws SQLException
  { conn.setTransactionIsolation(level); }
  public void setTypeMap(Map map) throws SQLException { conn.setTypeMap(map); }
}

/*
 * Local Variables: ***
 * compile-command: "(cd ../../; ant -emacs compile)" ***
 * End: ***
 */
