/* $Id: PoolDriverTest.java,v 1.3 2001/11/06 07:58:27 886014 Rel $
 * Copyright (c) 2001 TSUTSUMI Kikuo. ALL RIGHTS RESERVED.
 *
 * type ''javac -classpath .;../lib PoolDriverTest'' to run.
 */
import connectionpool.*;
import java.sql.*;
import java.util.Vector;

public class PoolDriverTest extends Thread
{
  static final String DBDRIVER = "connectionpool.ConnectionPoolDriver";
  static final String DBURL = "jdbc:connectionpool:connectionPoolTest";
  private long totalTime;
  private final static int THREAD_NUM = 5;
  private final static int TEST_NUM = 10;
  public static void main (String [] a)
  {
    Vector threads = new Vector (THREAD_NUM);
    PoolDriverTest ct;
    try
      {
	Class.forName (DBDRIVER);
	ConnectionPoolDriver d;
	d = (ConnectionPoolDriver) DriverManager.getDriver (DBURL);
	System.out.println ("ConnectionPoolDriver: Version " +
			    d.getMajorVersion () + "." +
			    d.getMinorVersion ());
	ConnectionPoolDriver.printProperties ();
	for (int i = 0; i < THREAD_NUM; i ++)
	  {
	    ct = new PoolDriverTest ();
	    ct.start ();
	    threads.add (ct);
	    try { sleep (100); } catch (InterruptedException e) {}
	  }
	long totalTime = 0;
	while (!threads.isEmpty ())
	  {
	    ct = (PoolDriverTest)threads.remove (threads.size () - 1);
	    try { ct.join (); } catch (InterruptedException e) {}
	    totalTime += ct.totalTime;
	  }
	ConnectionPoolDriver.clear ();
	System.out.println ("TOTAL AVLG: " +
			    ((float)totalTime / THREAD_NUM / TEST_NUM) +
			    " ms");
      }
    catch (Exception e)
      {
	e.printStackTrace ();
      }
  }
  public void run ()
  {
    Connection conn = null;
    Statement st = null;
    ResultSet rs = null;
    Vector conns = new Vector (TEST_NUM);
    Vector recs = new Vector (TEST_NUM);
    Vector laps = new Vector (TEST_NUM);
    long start = System.currentTimeMillis ();
    long lap;
    long end = start;
    try
      {
	for (int i = 0; i < TEST_NUM; i ++)
	  {
	    int count = 0;
	    conn = DriverManager.getConnection (DBURL);
	    conns.add (conn.toString ());
	    st = conn.createStatement ();
	    rs = st.executeQuery ("SELECT * FROM TBL ORDER BY ID DESC");
	    while (rs.next ())
	    {
	      String subj = rs.getString ("CLMN");
	      count ++;
	    }
	    recs.add (new Integer (count));
	    rs.close ();
	    rs = null;
	    st.close ();
	    st = null;
	    conn.close ();
	    conn = null;
	    lap = System.currentTimeMillis ();
	    laps.add (new Long (lap - end));
	    end = lap;
	  }
	end = System.currentTimeMillis ();
      }
    catch (Exception e)
      {
	System.out.print ("(" + getName () + ") ");
	e.printStackTrace ();
      }
    finally
      {
	try
	  {
	    if (conn != null) conn.close ();
	    if (st != null) st.close ();
	    if (rs != null) rs.close ();
	  }
	catch (Exception e) {}
      }
    String prompt;
    String msg;
    for (int i = 0; i < conns.size (); i ++)
      {
	prompt = "(" + getName () + ") " + i + ": ";
	msg = prompt + "connection: " + conns.get (i);
	System.out.println (msg);
	msg = prompt + "read " + recs.get (i) + " records";
	System.out.println (msg);
	msg = prompt + "lap: " + laps.get (i) + " ms";
	System.out.println (msg);
      }
    System.out.println ("(" + getName () + ") time: " +
			(end - start) + " ms");
    System.out.println ("(" + getName () + ") avlg: " +
			(float)(end - start) / laps.size () + " ms");
    totalTime = end - start;
  }
}

/*
 * Local Variables: ***
 * compile-command: "javac -classpath ../lib PoolDriverTest.java" ***
 * End: ***
 */
